@@ -183,7 +183,6 @@ update_output_settings (GvcMixerDialog *dialog)
 {
         GvcMixerStream *stream;
         GvcChannelMap  *map;
-        guint           num_settings;
         const GList    *ports;
 
         g_debug ("Updating output settings");
@@ -214,6 +213,11 @@ update_output_settings (GvcMixerDialog *dialog)
                 return;
         }
 
+        gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (dialog->priv->output_bar),
+                                         gvc_mixer_stream_get_base_volume (stream));
+        gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (dialog->priv->output_bar),
+                                          gvc_mixer_stream_get_can_decibel (stream));
+
         map = gvc_mixer_stream_get_channel_map (stream);
         if (map == NULL) {
                 g_warning ("Default sink stream has no channel map");
@@ -230,7 +234,6 @@ update_output_settings (GvcMixerDialog *dialog)
                             dialog->priv->output_balance_bar,
                             FALSE, FALSE, 12);
         gtk_widget_show (dialog->priv->output_balance_bar);
-        num_settings = 1;
 
         if (gvc_channel_map_can_fade (map)) {
                 dialog->priv->output_fade_bar = gvc_balance_bar_new (map, BALANCE_TYPE_FR);
@@ -243,7 +246,6 @@ update_output_settings (GvcMixerDialog *dialog)
                                     dialog->priv->output_fade_bar,
                                     FALSE, FALSE, 12);
                 gtk_widget_show (dialog->priv->output_fade_bar);
-                num_settings++;
         }
 
         if (gvc_channel_map_has_lfe (map)) {
@@ -257,7 +259,6 @@ update_output_settings (GvcMixerDialog *dialog)
                                     dialog->priv->output_lfe_bar,
                                     FALSE, FALSE, 12);
                 gtk_widget_show (dialog->priv->output_lfe_bar);
-                num_settings++;
         }
 
         ports = gvc_mixer_stream_get_ports (stream);
@@ -663,7 +664,9 @@ on_adjustment_value_changed (GtkAdjustment  *adjustment,
                 g_free (name);
 
                 /* FIXME would need to do that in the balance bar really... */
-                gvc_mixer_stream_set_is_muted (stream, volume == 0);
+                /* Make sure we do not unmute muted streams, there's a button for that */
+                if (volume == 0.0)
+                        gvc_mixer_stream_set_is_muted (stream, TRUE);
                 /* Only push the volume if it's actually changed */
                 if (gvc_mixer_stream_set_volume(stream, (pa_volume_t) rounded) != FALSE)
                         gvc_mixer_stream_push_volume (stream);
