@@ -26,13 +26,15 @@
 #define _ISOC99_SOURCE
 
 #include <math.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include "volume.h"
 #include "button.h"
-#include "stock.h"
+
+G_DEFINE_TYPE (GnomeVolumeControlVolume, gnome_volume_control_volume, GTK_TYPE_FIXED)
+
 
 static void	gnome_volume_control_volume_class_init	(GnomeVolumeControlVolumeClass *klass);
 static void	gnome_volume_control_volume_init	(GnomeVolumeControlVolume *el);
@@ -52,35 +54,6 @@ static void	cb_lock_toggled				(GtkToggleButton *button,
 
 static gboolean	cb_check				(gpointer   data);
 
-static GtkFixedClass *parent_class = NULL;
-
-GType
-gnome_volume_control_volume_get_type (void)
-{
-  static GType gnome_volume_control_volume_type = 0;
-
-  if (!gnome_volume_control_volume_type) {
-    static const GTypeInfo gnome_volume_control_volume_info = {
-      sizeof (GnomeVolumeControlVolumeClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gnome_volume_control_volume_class_init,
-      NULL,
-      NULL,
-      sizeof (GnomeVolumeControlVolume),
-      0,
-      (GInstanceInitFunc) gnome_volume_control_volume_init,
-      NULL
-    };
-
-    gnome_volume_control_volume_type =
-	g_type_register_static (GTK_TYPE_FIXED, 
-				"GnomeVolumeControlVolume",
-				&gnome_volume_control_volume_info, 0);
-  }
-
-  return gnome_volume_control_volume_type;
-}
 
 static void
 gnome_volume_control_volume_class_init (GnomeVolumeControlVolumeClass *klass)
@@ -88,8 +61,6 @@ gnome_volume_control_volume_class_init (GnomeVolumeControlVolumeClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_FIXED);
-
   gobject_class->dispose = gnome_volume_control_volume_dispose;
   gtkwidget_class->size_allocate = gnome_volume_control_volume_size_alloc;
   gtkwidget_class->size_request = gnome_volume_control_volume_size_req;
@@ -107,73 +78,9 @@ gnome_volume_control_volume_init (GnomeVolumeControlVolume *vol)
   vol->scales = NULL;
   vol->button = NULL;
   vol->locked = FALSE;
-  vol->appbar = NULL;
   vol->id = 0;
 }
 
-static gboolean
-cb_mouseover (GtkScale *scale,
-	      GdkEventCrossing *event,
-	      gpointer data)
-{
-  GnomeVolumeControlVolume *vol = data;
-  gchar *msg, *chan;
-  GList *items;
-  gint n = 0;
-
-  for (items = vol->scales; items != NULL; items = items->next, n++)
-    if (items->data == scale)
-      break;
-  g_assert (items != NULL);
-
-  /* somewhat dirty hack that will suffice for now. 1 chan
-   * means mono, two means stereo (left/right) and > 2 means
-   * alsa, where channel order is front, rear, center, lfe,
-   * side. */
-  if (vol->track->num_channels == 1) {
-    chan = _("mono");
-  } else if (vol->track->num_channels == 2) {
-    chan = (n == 0) ? _("left") : _("right");
-  } else {
-    switch (n) {
-      case 0:  chan = _("front left"); break;
-      case 1:  chan = _("front right"); break;
-      case 2:  chan = _("rear left"); break;
-      case 3:  chan = _("rear right"); break;
-      case 4:  chan = _("front center"); break;
-      /* Translators: This is the name of a surround sound channel. It
-       * stands for "Low-Frequency Effects". If you're not sure that
-       * this has an established and different translation in your
-       * language, leave it unchanged. */
-      case 5:  chan = _("LFE"); break;
-      case 6:  chan = _("side left"); break;
-      case 7:  chan = _("side right"); break;
-      default: chan = _("unknown"); break;
-    }
-  }
-
-  /* Here, we can actually tell people that this
-   * is a slider that will change channel X. */
-  msg = g_strdup_printf (_("Volume of %s channel on %s"),
-      chan, vol->track->label);
-  gnome_appbar_push (vol->appbar, msg);
-  g_free (msg);
-
-  return FALSE;
-}
-
-static gboolean
-cb_mouseout (GtkScale *scale,
-	     GdkEventCrossing *event,
-	     gpointer data)
-{
-  GnomeVolumeControlVolume *vol = data;
-
-  gnome_appbar_pop (vol->appbar);
-
-  return FALSE;
-}
-
 static GtkWidget *
 get_scale (GnomeVolumeControlVolume *vol,
 	   gint num_chan,
@@ -193,10 +100,6 @@ get_scale (GnomeVolumeControlVolume *vol,
   slider = gtk_vscale_new (GTK_ADJUSTMENT (adj));
   gtk_scale_set_draw_value (GTK_SCALE (slider), FALSE);
   gtk_range_set_inverted (GTK_RANGE (slider), TRUE);
-  g_signal_connect (slider, "enter-notify-event",
-		    G_CALLBACK (cb_mouseover), vol);
-  g_signal_connect (slider, "leave-notify-event",
-		    G_CALLBACK (cb_mouseout), vol);
 
   /* a11y */
   accessible = gtk_widget_get_accessible (slider);
@@ -231,7 +134,7 @@ get_button (GnomeVolumeControlVolume *vol,
   msg = g_strdup_printf (_("Lock channels for %s together"), vol->track->label);
   vol->button = gnome_volume_control_button_new ("chain.png",
 						 "chain-broken.png",
-						 vol->appbar, msg);
+						 msg);
   g_free (msg);
   g_signal_connect (vol->button, "clicked",
 		    G_CALLBACK (cb_lock_toggled), vol);
@@ -256,18 +159,21 @@ get_button (GnomeVolumeControlVolume *vol,
 GtkWidget *
 gnome_volume_control_volume_new (GstMixer *mixer,
 				 GstMixerTrack *track,
-				 gint      padding,
-				 GnomeAppBar *appbar)
+				 gint      padding)
 {
   GnomeVolumeControlVolume *vol;
   gint *volumes, n;
+  gchar *msg, *chan;
+  gboolean need_timeout = TRUE;
+
+  need_timeout = ((gst_mixer_get_mixer_flags (GST_MIXER (mixer)) &
+		   GST_MIXER_FLAG_AUTO_NOTIFICATIONS) == 0);
 
   /* volume */
   vol = g_object_new (GNOME_VOLUME_CONTROL_TYPE_VOLUME, NULL);
   gst_object_ref (GST_OBJECT (mixer));
   vol->mixer = mixer;
   vol->track = g_object_ref (G_OBJECT (track));
-  vol->appbar = appbar;
   if (padding >= 0)
     vol->padding = padding;
 
@@ -282,6 +188,39 @@ gnome_volume_control_volume_new (GstMixer *mixer,
     gtk_fixed_put (GTK_FIXED (vol), slider, 0, 0);
     gtk_widget_show (slider);
     vol->scales = g_list_append (vol->scales, slider);
+
+    /* somewhat dirty hack that will suffice for now. 1 chan
+     * means mono, two means stereo (left/right) and > 2 means
+     * alsa, where channel order is front, rear, center, lfe,
+     * side. */
+    if (vol->track->num_channels == 1) {
+        chan = _("mono");
+    } else if (vol->track->num_channels == 2) {
+        chan = (n == 0) ? _("left") : _("right");
+    } else {
+        switch (n) {
+            case 0:  chan = _("front left"); break;
+            case 1:  chan = _("front right"); break;
+            case 2:  chan = _("rear left"); break;
+            case 3:  chan = _("rear right"); break;
+            case 4:  chan = _("front center"); break;
+            /* Translators: This is the name of a surround sound channel. It
+             * stands for "Low-Frequency Effects". If you're not sure that
+             * this has an established and different translation in your
+             * language, leave it unchanged. */
+            case 5:  chan = _("LFE"); break;
+            case 6:  chan = _("side left"); break;
+            case 7:  chan = _("side right"); break;
+            default: chan = _("unknown"); break;
+        }
+    }
+
+    /* Here, we can actually tell people that this
+     * is a slider that will change channel X. */
+    msg = g_strdup_printf (_("Volume of %s channel on %s"),
+            chan, vol->track->label);
+    gtk_widget_set_tooltip_text (slider, msg);
+    g_free (msg);
   }
 
   /* chainbutton */
@@ -294,7 +233,8 @@ gnome_volume_control_volume_new (GstMixer *mixer,
   g_free (volumes);
 
   /* GStreamer signals */
-  vol->id = g_timeout_add (100, cb_check, vol);
+  if (need_timeout)
+    vol->id = g_timeout_add (100, cb_check, vol);
 
   return GTK_WIDGET (vol);
 }
@@ -324,7 +264,7 @@ gnome_volume_control_volume_dispose (GObject *object)
     vol->scales = NULL;
   }
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (gnome_volume_control_volume_parent_class)->dispose (object);
 }
 
 /*
@@ -337,7 +277,6 @@ gnome_volume_control_volume_size_req (GtkWidget *widget,
 {
   GnomeVolumeControlVolume *vol = GNOME_VOLUME_CONTROL_VOLUME (widget);
   GtkRequisition but_req, scale_req;
-  gint w,h;
 
   /* request size of kids */
   GTK_WIDGET_GET_CLASS (vol->button)->size_request (vol->button, &but_req);
@@ -359,7 +298,7 @@ gnome_volume_control_volume_size_alloc (GtkWidget *widget,
   GnomeVolumeControlVolume *vol = GNOME_VOLUME_CONTROL_VOLUME (widget);
   GtkRequisition but_req, scale_req;
   GtkAllocation but_all, scale_all;
-  gint x_offset, but_deco_y_offset, but_x_offset, but_deco_width, n = 0;
+  gint x_offset, but_deco_width, n = 0;
   GList *scales;
 
   /* loop? */
@@ -400,7 +339,7 @@ gnome_volume_control_volume_size_alloc (GtkWidget *widget,
   }
 
   /* parent will resize window */
-  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, alloc);
+  GTK_WIDGET_CLASS (gnome_volume_control_volume_parent_class)->size_allocate (widget, alloc);
 }
 
 static gboolean
@@ -453,7 +392,7 @@ gnome_volume_control_volume_expose (GtkWidget *widget,
   }
 
   /* take care of redrawing the kids */
-  return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, expose);
+  return GTK_WIDGET_CLASS (gnome_volume_control_volume_parent_class)->expose_event (widget, expose);
 }
 
 /*
@@ -491,16 +430,6 @@ cb_volume_changed (GtkAdjustment *_adj,
   vol->locked = FALSE;
 }
 
-/*
- * To be called after unmuting.
- */
-
-void
-gnome_volume_control_volume_sync (GnomeVolumeControlVolume * vol)
-{
-  cb_volume_changed (gtk_range_get_adjustment (vol->scales->data), vol);
-}
-
 static void
 cb_lock_toggled (GtkToggleButton *button,
 		 gpointer         data)
@@ -560,32 +489,21 @@ gnome_volume_control_volume_ask (GnomeVolumeControlVolume * vol,
   }
 }
 
-/*
- * Timeout to check for volume changes.
- */
 
-static gboolean
-cb_check (gpointer data)
+void
+gnome_volume_control_volume_update (GnomeVolumeControlVolume *vol)
 {
-  GnomeVolumeControlVolume *vol = data;
   gint *volumes, n;
-  gboolean real_zero, slider_zero;
   GList *scales;
 
   /* don't do callbacks */
   if (vol->locked)
-    return TRUE;
+    return;
+
   vol->locked = TRUE;
 
   volumes = g_new (gint, vol->track->num_channels);
   gst_mixer_get_volume (vol->mixer, vol->track, volumes);
-  gnome_volume_control_volume_ask (vol, &real_zero, &slider_zero);
-  if (real_zero || GST_MIXER_TRACK_HAS_FLAG (vol->track,
-					     GST_MIXER_TRACK_MUTE)) {
-    g_free (volumes);
-    vol->locked = FALSE;
-    return TRUE;
-  }
 
   /* did we change? */
   for (n = 0, scales = vol->scales;
@@ -599,12 +517,22 @@ cb_check (gpointer data)
     /* should we release lock? */
     if (volumes[n] != volumes[0]) {
       gnome_volume_control_button_set_active (
-          GNOME_VOLUME_CONTROL_BUTTON (vol->button), FALSE);
+	GNOME_VOLUME_CONTROL_BUTTON (vol->button), FALSE);
     }
   }
 
   g_free (volumes);
   vol->locked = FALSE;
+}
+
+/*
+ * Timeout to check for volume changes.
+ */
+
+static gboolean
+cb_check (gpointer data)
+{
+  gnome_volume_control_volume_update (data);
 
   return TRUE;
 }
